Documentation
¶
Index ¶
- Variables
- func ArgMax(s []float32) (int, float32, error)
- func CloseFile(file io.Closer) error
- func GetPathType(path string) string
- func Mean(vector []float32) float32
- func PathJoinSafe(elem ...string) string
- func ReadFileBytes(filename string) ([]byte, error)
- func SoftMax(vector []float32) []float32
- func SumSlice(s []float64) float64
- type AfsFS
- type AfsFile
- type AfsFileInfo
Constants ¶
This section is empty.
Variables ¶
var FileSystem = afs.New()
Functions ¶
func GetPathType ¶
func PathJoinSafe ¶
PathJoinSafe wrapper around filepath.Join to ensure that paths are correctly constructed if the path is a normal OS path, just use filepath.Join if the path is S3, trim any trailing slashes and construct it manually from the components so that double slashes (e.g. s3://) are preserved.
func ReadFileBytes ¶
Types ¶
type AfsFileInfo ¶
type AfsFileInfo struct {
// contains filtered or unexported fields
}
func (AfsFileInfo) IsDir ¶
func (fileInfo AfsFileInfo) IsDir() bool
func (AfsFileInfo) ModTime ¶
func (fileInfo AfsFileInfo) ModTime() time.Time
func (AfsFileInfo) Mode ¶
func (fileInfo AfsFileInfo) Mode() fs.FileMode
func (AfsFileInfo) Name ¶
func (fileInfo AfsFileInfo) Name() string
func (AfsFileInfo) Size ¶
func (fileInfo AfsFileInfo) Size() int64
func (AfsFileInfo) Sys ¶
func (fileInfo AfsFileInfo) Sys() any
Click to show internal directories.
Click to hide internal directories.